<?php

declare(strict_types=1);

namespace Dedi\SyliusEcotaxPlugin\Calculator;

use Sylius\Component\Addressing\Matcher\ZoneMatcherInterface;
use Sylius\Component\Addressing\Model\ZoneInterface;
use Sylius\Component\Core\Model\AddressInterface;
use Sylius\Component\Core\Model\ProductVariantInterface;
use Sylius\Component\Core\Model\Scope;
use Sylius\Component\Taxation\Calculator\CalculatorInterface;
use Sylius\Component\Taxation\Resolver\TaxRateResolverInterface;

final class ProductVariantEcotaxWithTaxCalculator implements ProductVariantEcotaxCalculatorInterface
{
    public function __construct(
        private ProductVariantEcotaxCalculatorInterface $productVariantEcotaxCalculator,
        private TaxRateResolverInterface $taxRateResolver,
        private CalculatorInterface $taxCalculator,
        private ZoneMatcherInterface $zoneMatcher,
    ) {
    }

    public function calculate(ProductVariantInterface $variant, array $context): int
    {
        $price = $this->productVariantEcotaxCalculator->calculate($variant, $context);

        $zone = null;
        if (array_key_exists('address', $context) && $context['address'] instanceof AddressInterface) {
            $address = $context['address'];
            $zone = $this->getTaxZone($address);
        }

        return $this->getPriceWithTax($variant, $price, [
            'zone' => $zone,
        ]);
    }

    private function getPriceWithTax(ProductVariantInterface $variant, int $price, array $parameters = []): int
    {
        $taxRate = $this->taxRateResolver->resolve($variant, $parameters);

        if (null === $taxRate) {
            return $price;
        }

        $taxAmount = $this->taxCalculator->calculate($price, $taxRate);

        if ($taxRate->isIncludedInPrice()) {
            return $price;
        }

        return (int) ($price + $taxAmount);
    }

    private function getTaxZone(AddressInterface $address): ?ZoneInterface
    {
        return $this->zoneMatcher->match($address, Scope::TAX);
    }
}
