<?php

declare(strict_types=1);

namespace Dedi\SyliusEcotaxPlugin\Processor;

use Dedi\Contracts\SyliusPluginSettings\SettingsProviderInterface;
use Dedi\SyliusEcotaxPlugin\Calculator\EcotaxCalculatorInterface;
use Dedi\SyliusEcotaxPlugin\Model\EcotaxAdjustmentsAwareInterface;
use Dedi\SyliusEcotaxPlugin\Model\EcotaxesAwareInterface;
use Sylius\Component\Channel\Context\ChannelContextInterface;
use Sylius\Component\Core\Model\ChannelInterface;
use Sylius\Component\Core\Model\ChannelPricingInterface;
use Sylius\Component\Core\Model\OrderInterface;
use Sylius\Component\Core\Model\OrderItem;
use Sylius\Component\Core\Resolver\TaxationAddressResolverInterface;
use Sylius\Component\Order\Factory\AdjustmentFactoryInterface;
use Sylius\Component\Order\Model\OrderInterface as BaseOrderInterface;
use Sylius\Component\Order\Model\OrderItemUnitInterface;
use Sylius\Component\Order\Processor\OrderProcessorInterface;
use Webmozart\Assert\Assert;

class EcotaxOrderProcessor implements OrderProcessorInterface
{
    public function __construct(
        private ChannelContextInterface $channelContext,
        private AdjustmentFactoryInterface $adjustmentFactory,
        private EcotaxCalculatorInterface $ecotaxCalculator,
        private EcotaxCalculatorInterface $ecotaxWithTaxCalculator,
        private TaxationAddressResolverInterface $taxationAddressResolver,
        private SettingsProviderInterface $settingsProvider,
    ) {
    }

    public function process(BaseOrderInterface $order): void
    {
        $settings = $this->settingsProvider->provide();

        if (!$settings->isEnabled()) {
            return;
        }

        Assert::isInstanceOf($order, OrderInterface::class);

        foreach ($order->getItems() as $item) {
            foreach ($item->getUnits() as $unit) {
                $unit->removeAdjustments(EcotaxAdjustmentsAwareInterface::ADJUSTMENT_TYPE_ECOTAX);

                $this->processOrderItemUnit($order, $unit);
            }
        }

        $order->recalculateItemsTotal();
    }

    private function processOrderItemUnit(OrderInterface $order, OrderItemUnitInterface $unit): void
    {
        /** @var ChannelInterface $channel */
        $channel = $this->channelContext->getChannel();

        /** @var OrderItem $orderItem */
        $orderItem = $unit->getOrderItem();

        /** @var ChannelPricingInterface|null $channelPricing */
        $channelPricing = $orderItem->getVariant()?->getChannelPricingForChannel($channel);
        Assert::isInstanceOf($channelPricing, EcotaxesAwareInterface::class);

        $ecotaxes = $channelPricing->getEcotaxes();
        $address = $this->taxationAddressResolver->getTaxationAddressFromOrder($order);

        foreach ($ecotaxes as $ecotax) {
            $ecotaxAmount = $this->ecotaxCalculator->calculate($ecotax, [
                'channel' => $channel,
                'address' => $address,
            ]);

            if (0 === $ecotaxAmount) {
                continue;
            }

            $ecotaxAmountWithTax = $this->ecotaxWithTaxCalculator->calculate($ecotax, [
                'channel' => $channel,
                'address' => $address,
            ]);

            $unit->addAdjustment(
                $this->adjustmentFactory->createWithData(
                    EcotaxAdjustmentsAwareInterface::ADJUSTMENT_TYPE_ECOTAX,
                    (string) $ecotax->getType()?->getLabel(),
                    $ecotaxAmount,
                    true,
                    ['label' => $ecotax->getType()?->getLabel(), 'amount_including_tax' => $ecotaxAmountWithTax],
                ),
            );
        }
    }
}
